# -*- coding: utf-8 -*-
import sys
PY3 = sys.version_info >= (3,)
if PY3:
    from io import StringIO, BytesIO as StreamIO
else:
    from StringIO import StringIO, StringIO as StreamIO
from os.path import abspath, dirname
from unittest import TestCase

SKOOLKIT_HOME = abspath(dirname(dirname(__file__)))
sys.path.insert(0, SKOOLKIT_HOME)
from skoolkit import bin2tap, skool2asm, skool2ctl, skool2html, skool2sft, sna2skool

class Stream:
    def __init__(self):
        self.buffer = StreamIO()
        self.text_buffer = StringIO()

    def write(self, text):
        self.text_buffer.write(text)

    def writelines(self, lines):
        self.text_buffer.writelines(lines)

    def flush(self):
        return

    def getvalue(self):
        if self.text_buffer.tell():
            return self.text_buffer.getvalue()
        return self.buffer.getvalue().decode('utf-8')

    def clear(self):
        self.buffer.seek(0)
        self.buffer.truncate()
        self.text_buffer.seek(0)
        self.text_buffer.truncate()

class SkoolKitTestCase(TestCase):
    def setUp(self):
        self.maxDiff = None
        self.stdout = sys.stdout
        self.stderr = sys.stderr
        sys.stdout = self.out = Stream()
        sys.stderr = self.err = Stream()
        self.remove_files()

    def tearDown(self):
        self.remove_files()
        sys.stdout = self.stdout
        sys.stderr = self.stderr

    def clear_streams(self):
        self.out.clear()
        self.err.clear()

    def remove_files(self):
        return

    def assert_output_equal(self, output, expected, prefixes=False):
        for i, line in enumerate(expected):
            if i < len(output):
                if prefixes:
                    self.assertEqual(output[i][:len(line)], line)
                else:
                    self.assertEqual(output[i], line)
        if len(output) > len(expected):
            self.fail("Unexpected extra line in output: '%s'" % output[len(expected)])
        elif len(expected) > len(output):
            self.fail("Expected lines missing from output, starting with: '%s'" % expected[len(output)])

    def to_lines(self, text, strip_cr):
        # Use rstrip() to remove '\r' characters (useful on Windows)
        if strip_cr:
            lines = [line.rstrip() for line in text.split('\n')]
        else:
            lines = text.split('\n')
        if lines[-1] == '':
            lines.pop()
        return lines

    def _run_skoolkit_command(self, cmd, args, out_lines, err_lines, strip_cr):
        self.clear_streams()
        try:
            cmd(args.split())
        except SystemExit:
            pass
        out = self.to_lines(self.out.getvalue(), strip_cr) if out_lines else self.out.getvalue()
        err = self.to_lines(self.err.getvalue(), strip_cr) if err_lines else self.err.getvalue()
        return out, err

    def run_bin2tap(self, args='', out_lines=True, err_lines=False, strip_cr=True):
        return self._run_skoolkit_command(bin2tap.main, args, out_lines, err_lines, strip_cr)

    def run_skool2asm(self, args='', out_lines=True, err_lines=False, strip_cr=True):
        return self._run_skoolkit_command(skool2asm.main, args, out_lines, err_lines, strip_cr)

    def run_skool2ctl(self, args='', out_lines=True, err_lines=False, strip_cr=True):
        return self._run_skoolkit_command(skool2ctl.main, args, out_lines, err_lines, strip_cr)

    def run_skool2html(self, args='', out_lines=True, err_lines=False, strip_cr=True):
        return self._run_skoolkit_command(skool2html.main, args, out_lines, err_lines, strip_cr)

    def run_skool2sft(self, args='', out_lines=True, err_lines=False, strip_cr=True):
        return self._run_skoolkit_command(skool2sft.main, args, out_lines, err_lines, strip_cr)

    def run_sna2skool(self, args='', out_lines=True, err_lines=False, strip_cr=True):
        return self._run_skoolkit_command(sna2skool.main, args, out_lines, err_lines, strip_cr)
